# Hybrid demultiplexing + LAA pipeline
# repalces pb_barcode2_laa

version 1.0

import "tasks/barcoding.wdl" as barcoding
import "pb_demux_subreads.wdl" as pb_demux_subreads
import "pb_laa.wdl" as pb_laa

workflow dev_barcode_laa {
  input {
    File eid_subread
    File eid_barcode
    String new_dataset_name = "test_dev_barcode_laa"
    Boolean lima_symmetric_barcodes = true
    Boolean lima_peek_guess = true
    Int lima_min_score = 0
    Boolean use_barcode_uuids = false
    Boolean pb_test_mode = false
    Int min_bq_filter = 26
    String dataset_filters = ""
    Boolean laa_chimera_filter = true
    Boolean laa_clustering = true
    Boolean laa_phasing = true
    Boolean laa_full_length = false
    Int laa_min_length = 3000
    Int laa_max_reads = 2000
    Float laa_min_snr = 2.5
    Float laa_min_read_score = 0.75
    Int laa_min_barcode_score = 26
    Int laa_max_phasing_reads = 500

    Int nproc = 1
    Int max_nchunks = 100
    String log_level = "INFO"
    String? tmp_dir
  }

  # This takes care of absolutization
  call pb_demux_subreads.pb_demux_subreads {
    input:
      eid_subread = eid_subread,
      eid_barcode = eid_barcode,
      new_dataset_name = new_dataset_name,
      lima_symmetric_barcodes = lima_symmetric_barcodes,
      lima_peek_guess = lima_peek_guess,
      lima_min_score = lima_min_score,
      use_barcode_uuids = use_barcode_uuids,
      pb_test_mode = pb_test_mode,
      min_bq_filter = min_bq_filter,
      nproc = nproc,
      log_level = log_level
  }

  call barcoding.datastore_to_merged_dataset {
    input:
      datastore = pb_demux_subreads.barcoded_reads,
      dataset_ext = ".subreadset.xml",
      nproc = nproc,
      log_level = log_level
  }

  call pb_laa.pb_laa {
    input:
      eid_subread = datastore_to_merged_dataset.mergedxml,
      laa_chimera_filter = laa_chimera_filter,
      laa_clustering = laa_clustering,
      laa_phasing = laa_phasing,
      laa_full_length = laa_full_length,
      laa_min_length = laa_min_length,
      laa_max_reads = laa_max_reads,
      laa_min_snr = laa_min_snr,
      laa_min_read_score = laa_min_read_score,
      laa_min_barcode_score = laa_min_barcode_score,
      laa_max_phasing_reads = laa_max_phasing_reads,
      nproc = nproc,
      log_level = log_level
  }

  output {
    File barcoded_reads = pb_demux_subreads.barcoded_reads
    File report_barcodes = pb_demux_subreads.report_barcodes
    File barcoded_reports_datastore = pb_demux_subreads.barcoded_reports_datastore
    File consensus_fastq = pb_laa.consensus_fastq
    File chimeras_fastq = pb_laa.chimeras_fastq
    File summary_csv = pb_laa.summary_csv
    File report_consensus = pb_laa.report_consensus
    File report_input = pb_laa.report_input
  }
}
