version 1.0

import "tasks/pbreports.wdl" as pbreports
import "dev/tasks.wdl" as dev_tasks
import "tasks/sl_tasks.wdl" as sl_tasks
import "tasks/pbcoretools.wdl" as pbcoretools

workflow dev_diagnostic_subreads {
  input {
    File eid_subread
    File? eid_ref_dataset
    Int? exit_code
    Int sleep_time = 0
    Boolean emit_warn_alarm = false
    Boolean emit_error_alarm = false

    Int nproc = 1
    Int max_nchunks = 100
    String log_level = "INFO"
    String? tmp_dir
  }

  call dev_tasks.dump_environment {
    input:
      sleep_time = sleep_time,
      nproc = 1,
      log_level = "INFO"
  }

  call pbcoretools.dataset_absolutize as update_subreads {
    input:
      dataset = eid_subread,
      log_level = log_level,
      nproc = 1
  }

  call pbreports.subread_stats {
    input:
      subreads = update_subreads.updated_xml,
      nproc = nproc,
      log_level = log_level
  }

  call dev_tasks.samples_report {
    input:
      dataset = update_subreads.updated_xml,
      nproc = nproc,
      log_level = log_level
  }

  call sl_tasks.simple_dataset_report {
    input:
      dataset = update_subreads.updated_xml,
      nproc = nproc,
      log_level = log_level
  }

  if (defined(exit_code)) {
    call dev_tasks.dev_failing_task {
      input:
        exit_code = exit_code,
        nproc = nproc,
        log_level = log_level
    }
  }

  if (defined(eid_ref_dataset)) {

    call pbcoretools.dataset_absolutize as update_reference {
      input:
        dataset = eid_ref_dataset,
        log_level = log_level,
        nproc = 1
    }

    call sl_tasks.simple_dataset_report as reference_report {
      input:
        dataset = update_reference.updated_xml,
        nproc = nproc,
        log_level = log_level
    }
  }

  if (emit_warn_alarm) {
    call dev_tasks.emit_alarm {
      input:
        dataset = eid_subread
    }
  }

  if (emit_error_alarm) {
    call dev_tasks.raise_alarm {
      input:
        dataset = eid_subread
    }
  }

  output {
    File report_subreads = subread_stats.report
    File? report_dataset = simple_dataset_report.report
    File report_samples = samples_report.report
    File? report_reference = reference_report.report
  }
}
