# Mock demultiplexing workflow for testing scaling and end-to-end integration

version 1.0

import "dev/tasks.wdl" as dev_tasks
import "tasks/pbcoretools.wdl" as pbcoretools

workflow dev_mock_demux {
  input {
    File eid_subread
    File eid_barcode
    Boolean use_barcode_uuids = false

    Int nproc = 1
    Int max_nchunks = 100
    String log_level = "INFO"
    String? tmp_dir
  }

  call pbcoretools.dataset_absolutize as update_subreads {
    input:
      dataset = eid_subread,
      log_level = log_level,
      nproc = 1
  }

  call pbcoretools.dataset_absolutize as update_barcodes {
    input:
      dataset = eid_barcode,
      log_level = log_level,
      nproc = 1
  }

  call dev_tasks.mock_demux {
    input:
      dataset = update_subreads.updated_xml,
      dataset_ext = ".subreadset.xml",
      log_level = log_level,
      nproc = 1
  }

  call dev_tasks.mock_update_barcoded_sample_metadata {
    input:
      lima_datastore = mock_demux.datastore,
      input_reads = update_subreads.updated_xml,
      barcodes = update_barcodes.updated_xml,
      use_barcode_uuids = use_barcode_uuids,
      nproc = 1,
      log_level = log_level
  }

  output {
    File datastore = mock_update_barcoded_sample_metadata.datastore
  }
}
