# dev workflow for testing behavior of dataset parentage
version 1.0

import "tasks/pbcoretools.wdl" as pbcoretools

task make_new_parent {
  input {
    File subreads
  }
  command {
    cp ${subreads} new_parent.subreadset.xml
    dataset newuuid new_parent.subreadset.xml --random
  }
  runtime {
    backend: "Local"
    cpu: 1
  }
  output {
    File parent = "new_parent.subreadset.xml"
  }
}

task make_child {
  input {
    File parent
  }
  command <<<
    python <<EOF
    from pbcore.io import SubreadSet
    ds = SubreadSet("~{parent}")
    ds.metadata.addParentDataSet(ds.uuid,
                                 ds.datasetType,
                                 createdBy="AnalysisJob",
                                 timeStampedName="")
    ds.newUuid(random=True)
    ds.name = ds.name + " (child)"
    ds.write("child.subreadset.xml")
    EOF
  >>>
  runtime {
    backend: "Local"
    cpu: 1
  }
  output {
    File child = "child.subreadset.xml"
  }
}

workflow dev_mock_reparent {
  input {
    File eid_subread

    Int nproc = 1
    Int max_nchunks = 1
    String log_level = "INFO"
    String? tmp_dir
  }

  call pbcoretools.dataset_absolutize as update_subreads {
    input:
      dataset = eid_subread,
      log_level = log_level,
      nproc = 1
  }

  call make_new_parent {
    input:
      subreads = update_subreads.updated_xml
  }

  call make_child as make_child_1 {
    input:
      parent = update_subreads.updated_xml
  }

  call make_child as make_child_2 {
    input:
      parent = make_new_parent.parent
  }

  output {
    # this file will have a parent UUID known to smrtlink
    File child_1 = make_child_1.child
    # this file will have a randomly generated parent UUID
    File child_2 = make_child_2.child
  }
}
