# Alignment starting from ConsensusReadSet

version 1.0

import "wf_mapping.wdl"
import "tasks/pbcoretools.wdl" as pbcoretools

workflow pb_align_ccs {
  input {
    File eid_ccs
    File eid_ref_dataset
    String? mapping_biosample_name
    Float mapping_min_concordance = 70
    Int mapping_min_length = 50
    Boolean consolidate_aligned_bam = false
    String? mapping_pbmm2_overrides

    Int nproc = 1
    Int max_nchunks = 100
    String log_level = "INFO"
    String? tmp_dir
  }

  call pbcoretools.dataset_absolutize as update_ccs {
    input:
      dataset = eid_ccs,
      nproc = nproc,
      log_level = log_level
  }

  call pbcoretools.dataset_absolutize as update_reference {
    input:
      dataset = eid_ref_dataset,
      log_level = log_level,
      nproc = 1
  }

  call wf_mapping.mapping as ccs_mapping {
    input:
      reads = update_ccs.updated_xml,
      reference = update_reference.updated_xml,
      alignment_ext = ".consensusalignmentset.xml",
      mapping_stats_module = "mapping_stats_ccs",
      biosample_name = mapping_biosample_name,
      preset_mode = "CCS",
      min_concordance = mapping_min_concordance,
      min_length = mapping_min_length,
      pbmm2_overrides = mapping_pbmm2_overrides,
      run_bam_auto_consolidation = true,
      consolidate_aligned_bam = consolidate_aligned_bam,
      nproc = nproc,
      max_nchunks = max_nchunks,
      log_level = log_level,
      tmp_dir = tmp_dir
  }

  output {
    File mapped = ccs_mapping.mapped
    File? coverage_gff = ccs_mapping.coverage_gff
    File? report_mapping_stats = ccs_mapping.report_mapping_stats
    File? report_coverage = ccs_mapping.report_coverage
    File? mapped_bam_datastore = ccs_mapping.mapped_bam_datastore
  }
}
