# old pbalign mapping - OBSOLETE but still required for Primary workflows

version 1.0

import "tasks/pbcoretools.wdl" as pbcoretools
import "tasks/chunking.wdl" as chunking
import "tasks/pbreports.wdl" as pbreports

task pbalign {
  input {
    File subreads_xml
    File reference
    Float min_concordance = 70.0
    Int min_length = 50
    String pbalign_args = ""

    Int nproc
    String log_level = "DEBUG"
    String tmp_dir = "/tmp"
  }
  command {
    TMPDIR="${tmp_dir}" \
    pbalign \
      --nproc ${nproc} \
      --log-level ${log_level} \
      --minAccuracy ${min_concordance} \
      --minLength ${min_length} \
      ${pbalign_args} \
      ${subreads_xml} \
      ${reference} \
      mapped.alignmentset.xml
  }
  runtime {
    cpu: nproc
  }
  output {
    File mapped = "mapped.alignmentset.xml"
  }
}

workflow pb_align_pbalign {
  input {
    File eid_subread
    File eid_ref_dataset
    String dataset_filters = ""
    Int downsample_factor = 0
    Float mapping_min_concordance = 70
    Int mapping_min_length = 50
    String pbalign_args = ""

    Int nproc = 8
    String log_level = "INFO"
    Int max_nchunks = 100
    String? tmp_dir
  }

  call pbcoretools.dataset_absolutize as update_subreads {
    input:
      dataset = eid_subread,
      log_level = log_level,
      nproc = 1
  }

  call pbcoretools.dataset_absolutize as update_reference {
    input:
      dataset = eid_ref_dataset,
      log_level = log_level,
      nproc = 1
  }

  call pbcoretools.dataset_filter {
    input:
      dataset = update_subreads.updated_xml,
      filters = dataset_filters,
      downsample_factor = downsample_factor,
      nproc = 1,
      log_level = log_level
  }

  call chunking.split_dataset as split_reads {
    input:
      ds_in = dataset_filter.filtered,
      max_nchunks = max_nchunks,
      log_level = log_level
  }

  scatter (chunk in split_reads.chunks) {
    call pbalign {
      input:
        subreads_xml = chunk,
        reference = update_reference.updated_xml,
        min_concordance = mapping_min_concordance,
        min_length = mapping_min_length,
        pbalign_args = pbalign_args,
        nproc = nproc,
        log_level = log_level,
        tmp_dir = tmp_dir
    }
  }

  call chunking.gather_datasets as gather_alignments {
    input:
      chunks = pbalign.mapped,
      dataset_name = "'Mapped Reads'",
      dataset_ext = ".alignmentset.xml",
      log_level = log_level
  }

  call pbreports.mapping_stats as mapping_stats {
    input:
      mapped = gather_alignments.gathered,
      unmapped = dataset_filter.filtered,
      nproc = nproc,
      log_level = log_level
  }

  call pbreports.summarize_coverage as summarize_coverage {
    input:
      mapped = gather_alignments.gathered,
      reference = update_reference.updated_xml,
      nproc = nproc
  }

  call pbreports.coverage as coverage_report {
    input:
      reference = update_reference.updated_xml,
      coverage_gff = summarize_coverage.coverage_gff,
      nproc = nproc
  }

  output {
    File mapped = gather_alignments.gathered
    File coverage_gff = summarize_coverage.coverage_gff
    File report_mapping_stats = mapping_stats.report
    File report_coverage = coverage_report.report
  }
}
