# elisgnment starting from SubreadSet

version 1.0

import "wf_mapping.wdl"
import "tasks/pbcoretools.wdl" as pbcoretools

workflow pb_align_subreads {
  input {
    File eid_subread
    File eid_ref_dataset
    String dataset_filters = ""
    Int downsample_factor = 0
    String? mapping_biosample_name
    Float mapping_min_concordance = 70
    Int mapping_min_length = 50
    Boolean mapping_hq_mode = false
    Boolean mapping_zmw_mode = false
    String mapping_preset_mode = "SUBREAD"
    String? mapping_pbmm2_overrides

    Int nproc = 1
    String log_level = "INFO"
    Int max_nchunks = 100
    String? tmp_dir
  }

  call pbcoretools.dataset_absolutize as update_subreads {
    input:
      dataset = eid_subread,
      log_level = log_level,
      nproc = 1
  }

  call pbcoretools.dataset_absolutize as update_reference {
    input:
      dataset = eid_ref_dataset,
      log_level = log_level,
      nproc = 1
  }

  call pbcoretools.dataset_filter {
    input:
      dataset = update_subreads.updated_xml,
      filters = dataset_filters,
      downsample_factor = downsample_factor,
      nproc = 1,
      log_level = log_level
  }

  call wf_mapping.mapping {
    input:
      reads = dataset_filter.filtered,
      reference = update_reference.updated_xml,
      biosample_name = mapping_biosample_name,
      min_concordance = mapping_min_concordance,
      min_length = mapping_min_length,
      hq_mode = mapping_hq_mode,
      zmw_mode = mapping_zmw_mode,
      pbmm2_overrides = mapping_pbmm2_overrides,
      preset_mode = mapping_preset_mode,
      nproc = nproc,
      max_nchunks = max_nchunks,
      log_level = log_level,
      tmp_dir = tmp_dir
  }

  output {
    File mapped = mapping.mapped
    File? report_mapping_stats = mapping.report_mapping_stats
    File? coverage_gff = mapping.coverage_gff
    File? report_coverage = mapping.report_coverage
  }
}
