version 1.0

import "tasks/bam2fastx.wdl" as bam2fastx
import "tasks/pbcoretools.wdl" as pbcoretools

workflow pb_bam2fastx {
  input {
    File eid_subread
    String dataset_filters = ""
    Int downsample_factor = 0

    # Included here for consistency but this is not a parallel workflow
    Int max_nchunks = 1
    Int nproc = 1
    String log_level = "INFO"
    String? tmp_dir
  }

  call pbcoretools.dataset_absolutize as update_subreads {
    input:
      dataset = eid_subread,
      log_level = log_level,
      nproc = 1
  }

  call pbcoretools.dataset_filter {
    input:
      dataset = update_subreads.updated_xml,
      filters = dataset_filters,
      downsample_factor = downsample_factor,
      nproc = 1,
      log_level = log_level
  }

  call bam2fastx.bam2fasta_archive {
    input:
      reads = dataset_filter.filtered
  }

  call bam2fastx.bam2fastq_archive {
    input:
      reads = dataset_filter.filtered
  }

  output {
    File fasta_zip = bam2fasta_archive.fasta_zip
    File fastq_zip = bam2fastq_archive.fastq_zip
  }
}
