version 1.0

import "tasks/pbcoretools.wdl" as pbcoretools
import "tasks/chunking.wdl" as chunking
import "tasks/pbreports.wdl" as pbreports
import "tasks/unanimity.wdl" as unanimity

workflow pb_ccs {
  input {
    File eid_subread
    Float ccs_min_predicted_accuracy = 0.99
    Int ccs_min_passes = 3
    Float ccs_min_snr = 2.5
    Int ccs_max_poa_coverage = 0
    Int ccs_min_length = 10
    Int ccs_max_length = 50000
    Boolean ccs_polish = true
    Boolean ccs_use_run_design_uuid = false
    String? ccs_model_args
    Boolean ccs_by_strand = false
    String dataset_filters = ""
    Int downsample_factor = 0

    Int nproc = 1
    Int max_nchunks = 100
    Int target_size = 20000
    String log_level = "INFO"
    String? tmp_dir
  }

  call pbcoretools.dataset_absolutize as update_subreads {
    input:
      dataset = eid_subread,
      log_level = log_level,
      nproc = 1
  }

  call pbcoretools.dataset_filter {
    input:
      dataset = update_subreads.updated_xml,
      filters = dataset_filters,
      downsample_factor = downsample_factor,
      nproc = 1
  }

  call chunking.split_dataset as split_subreads {
    input:
      ds_in = dataset_filter.filtered,
      max_nchunks = max_nchunks,
      target_size = target_size
  }

  scatter (subreads in split_subreads.chunks) {
    call unanimity.ccs {
      input:
        subreads = subreads,
        min_passes = ccs_min_passes,
        min_predicted_accuracy = ccs_min_predicted_accuracy,
        min_snr = ccs_min_snr,
        max_poa_coverage = ccs_max_poa_coverage,
        min_length = ccs_min_length,
        max_length = ccs_max_length,
        model_args = ccs_model_args,
        polishing = ccs_polish,
        by_strand = ccs_by_strand,
        nproc = nproc,
        log_level = log_level
    }
  }

  call chunking.gather_datasets as gather_ccsxml {
    input:
      chunks = ccs.ccsxml,
      dataset_name = "'CCS Data'",
      dataset_ext = ".consensusreadset.xml",
      prefix = "ccs_reads",
      no_sub_datasets = true
  }

  call pbcoretools.update_consensus_reads {
    input:
      ccs_in = gather_ccsxml.gathered,
      subreads_in = update_subreads.updated_xml,
      use_run_design_uuid = ccs_use_run_design_uuid,
      nproc = nproc,
      log_level = log_level
  }

  call pbreports.ccs2 as ccs2_report {
    input:
      ccsxml = update_consensus_reads.ccsxml,
      nproc = nproc,
      log_level = log_level
  }

  call pbcoretools.auto_ccs_outputs as export_bam {
    input:
      ccsxml = update_consensus_reads.ccsxml,
      mode = "consolidate",
      nproc = nproc,
      log_level = log_level
  }

  call pbcoretools.auto_ccs_outputs as export_fasta {
    input:
      ccsxml = update_consensus_reads.ccsxml,
      mode = "fasta",
      nproc = nproc,
      log_level = log_level
  }

  call pbcoretools.auto_ccs_outputs as export_fastq {
    input:
      ccsxml = update_consensus_reads.ccsxml,
      mode = "fastq",
      nproc = nproc,
      log_level = log_level
  }

  output {
    File ccsxml = update_consensus_reads.ccsxml
    File report_ccs = ccs2_report.report
    File ccs_csv = ccs2_report.csv
    File bam_datastore = export_bam.datastore
    File fasta_datastore = export_fasta.datastore
    File fastq_datastore = export_fastq.datastore
  }
}
