version 1.0

import "pb_ccs.wdl" as ccs
import "wf_demultiplex.wdl" as demultiplex
import "tasks/pbcoretools.wdl" as pbcoretools

workflow pb_ccs_demux_auto {
  input {
    File eid_subread
    File eid_barcode
    Float ccs_min_predicted_accuracy = 0.99
    Int ccs_min_passes = 3
    Float ccs_min_snr = 2.5
    Int ccs_max_poa_coverage = 0
    Int ccs_min_length = 10
    Int ccs_max_length = 50000
    Boolean ccs_use_run_design_uuid = true
    String dataset_filters = ""
    Int downsample_factor = 0
    Boolean lima_symmetric_barcodes = true
    Boolean lima_peek_guess = true
    Int lima_min_score = 0
    Boolean lima_write_unbarcoded = true
    Boolean use_barcode_uuids = true
    Int min_bq_filter = 26
    Boolean lima_ignore_biosamples = false
    Boolean pb_test_mode = false

    Int nproc
    Int max_nchunks = 100
    String log_level = "INFO"
    String? tmp_dir
  }

  # option not exposed in SMRTLink
  Boolean lima_use_ccs = true

  call pbcoretools.dataset_absolutize as update_barcodes {
    input:
      dataset = eid_barcode,
      log_level = log_level,
      nproc = 1
  }

  call ccs.pb_ccs {
    input:
      # The pb_ccs workflow will take care of path absolutization
      eid_subread = eid_subread,
      ccs_min_predicted_accuracy = ccs_min_predicted_accuracy,
      ccs_min_passes = ccs_min_passes,
      ccs_min_snr = ccs_min_snr,
      ccs_max_poa_coverage = ccs_max_poa_coverage,
      ccs_min_length = ccs_min_length,
      ccs_max_length = ccs_max_length,
      ccs_use_run_design_uuid = ccs_use_run_design_uuid,
      dataset_filters = dataset_filters,
      downsample_factor = downsample_factor,
      nproc = nproc,
      log_level = log_level,
      tmp_dir = tmp_dir
  }

  call demultiplex.demultiplex_barcodes {
    input:
      unbarcoded_reads = pb_ccs.ccsxml,
      barcodes = update_barcodes.updated_xml,
      lima_symmetric_barcodes = lima_symmetric_barcodes,
      lima_peek_guess = lima_peek_guess,
      lima_min_score = lima_min_score,
      lima_write_unbarcoded = lima_write_unbarcoded,
      lima_use_ccs = lima_use_ccs,
      use_barcode_uuids = use_barcode_uuids,
      min_bq_filter = min_bq_filter,
      lima_ignore_biosamples = lima_ignore_biosamples,
      report_name = "barcode_ccs",
      pb_test_mode = pb_test_mode,
      nproc = nproc,
      log_level = log_level
  }

  call pbcoretools.auto_ccs_outputs_barcoded {
    input:
      datastore_in = demultiplex_barcodes.barcoded_reads,
      nproc = nproc,
      log_level = log_level
  }

  output {
    File ccsxml = pb_ccs.ccsxml
    File report_ccs = pb_ccs.report_ccs
    File ccs_csv = pb_ccs.ccs_csv
    File bam_datastore = pb_ccs.bam_datastore
    File fasta_datastore = pb_ccs.fasta_datastore
    File fastq_datastore = pb_ccs.fastq_datastore
    File barcoded_reads = demultiplex_barcodes.barcoded_reads
    File report_barcodes = demultiplex_barcodes.report_barcodes
    File summary_csv = demultiplex_barcodes.summary_csv
    File barcoded_reports_datastore = demultiplex_barcodes.barcoded_reports_datastore
    File demuxed_files_datastore = auto_ccs_outputs_barcoded.datastore
    File unbarcoded = demultiplex_barcodes.unbarcoded
  }
}
