version 1.0

import "pb_ccs.wdl" as ccs_wdl
import "wf_mapping.wdl"
import "tasks/pbcoretools.wdl" as pbcoretools
import "tasks/chunking.wdl" as chunking
import "tasks/pbreports.wdl" as pbreports
import "tasks/unanimity.wdl" as unanimity

workflow pb_ccs_mapping {
  input {
    File eid_subread
    File eid_ref_dataset
    Float ccs_min_predicted_accuracy = 0.99
    Int ccs_min_passes = 3
    Float ccs_min_snr = 2.5
    Int ccs_max_poa_coverage = 0
    Int ccs_min_length = 10
    Int ccs_max_length = 50000
    Boolean ccs_polish = true
    String? ccs_model_args
    String dataset_filters = ""
    Int downsample_factor = 0
    Boolean consolidate_aligned_bam = false
    Float mapping_min_concordance = 70
    Int mapping_min_length = 50
    String? mapping_biosample_name
    String? mapping_pbmm2_overrides

    Int nproc = 1
    Int max_nchunks = 100
    String log_level = "INFO"
    String? tmp_dir
  }

  call pbcoretools.dataset_absolutize as update_reference {
    input:
      dataset = eid_ref_dataset,
      log_level = log_level,
      nproc = 1
  }

  call ccs_wdl.pb_ccs {
    input:
      eid_subread = eid_subread,
      ccs_min_passes = ccs_min_passes,
      ccs_min_predicted_accuracy = ccs_min_predicted_accuracy,
      ccs_min_snr = ccs_min_snr,
      ccs_max_poa_coverage = ccs_max_poa_coverage,
      ccs_min_length = ccs_min_length,
      ccs_max_length = ccs_max_length,
      ccs_polish = ccs_polish,
      ccs_model_args = ccs_model_args,
      dataset_filters = dataset_filters,
      downsample_factor = downsample_factor,
      nproc = nproc,
      max_nchunks = max_nchunks,
      log_level = log_level
  }

  call wf_mapping.mapping as ccs_mapping {
    input:
      reads = pb_ccs.ccsxml,
      reference = update_reference.updated_xml,
      alignment_ext = ".consensusalignmentset.xml",
      mapping_stats_module = "mapping_stats_ccs",
      biosample_name = mapping_biosample_name,
      preset_mode = "CCS",
      min_concordance = mapping_min_concordance,
      min_length = mapping_min_length,
      pbmm2_overrides = mapping_pbmm2_overrides,
      run_bam_auto_consolidation = true,
      consolidate_aligned_bam = consolidate_aligned_bam,
      nproc = nproc,
      max_nchunks = max_nchunks,
      log_level = log_level,
      tmp_dir = tmp_dir
  }

  output {
    File ccsxml = pb_ccs.ccsxml
    File report_ccs = pb_ccs.report_ccs
    File ccs_csv = pb_ccs.ccs_csv
    File bam_datastore = pb_ccs.bam_datastore
    File fasta_datastore = pb_ccs.fasta_datastore
    File fastq_datastore = pb_ccs.fastq_datastore
    File mapped = ccs_mapping.mapped
    # these will always be generated here but need optional types because of
    # mapping workflow structure
    File? coverage_gff = ccs_mapping.coverage_gff
    File? report_mapping_stats = ccs_mapping.report_mapping_stats
    File? report_coverage = ccs_mapping.report_coverage
    # these are truly optional
    File? mapped_bam_datastore = ccs_mapping.mapped_bam_datastore
  }
}
