version 1.0

import "wf_demultiplex.wdl" as demultiplex
import "tasks/barcoding.wdl" as barcoding
import "tasks/pbcoretools.wdl" as pbcoretools

workflow pb_demux_subreads {
  input {
    File eid_subread
    File eid_barcode
    String new_dataset_name
    Boolean lima_symmetric_barcodes = true
    Boolean lima_peek_guess = true
    Int lima_min_score = 0
    Boolean lima_write_unbarcoded = true
    Boolean use_barcode_uuids = false
    Boolean pb_test_mode = false
    Int min_bq_filter = 26
    Boolean lima_ignore_biosamples = false

    Int nproc
    Int max_nchunks = 100
    String log_level = "INFO"
    String? tmp_dir
  }

  # option not exposed in SMRTLink
  Boolean lima_use_ccs = false

  call pbcoretools.dataset_absolutize as update_subreads {
    input:
      dataset = eid_subread,
      log_level = log_level,
      nproc = 1
  }

  call pbcoretools.dataset_absolutize as update_barcodes {
    input:
      dataset = eid_barcode,
      log_level = log_level,
      nproc = 1
  }

  call barcoding.reparent_dataset {
    input:
      input_reads = update_subreads.updated_xml,
      dataset_name = new_dataset_name,
      dataset_ext = ".subreadset.xml",
      nproc = nproc,
      log_level = log_level
  }

  call demultiplex.demultiplex_barcodes {
    input:
      unbarcoded_reads = reparent_dataset.new_parent,
      barcodes = update_barcodes.updated_xml,
      lima_symmetric_barcodes = lima_symmetric_barcodes,
      lima_peek_guess = lima_peek_guess,
      lima_min_score = lima_min_score,
      lima_write_unbarcoded = lima_write_unbarcoded,
      lima_use_ccs = lima_use_ccs,
      use_barcode_uuids = use_barcode_uuids,
      min_bq_filter = min_bq_filter,
      lima_ignore_biosamples = lima_ignore_biosamples,
      pb_test_mode = pb_test_mode,
      nproc = nproc,
      max_nchunks = max_nchunks,
      log_level = log_level
  }

  output {
    File input_reads = reparent_dataset.new_parent
    File barcoded_reads = demultiplex_barcodes.barcoded_reads
    File report_barcodes = demultiplex_barcodes.report_barcodes
    File summary_csv = demultiplex_barcodes.summary_csv
    File barcoded_reports_datastore = demultiplex_barcodes.barcoded_reports_datastore
    File unbarcoded = demultiplex_barcodes.unbarcoded
  }
}
