# Long Amplicon Analysis (LAA)
#
# TODO optimize chunking

version 1.0

import "tasks/pbcoretools.wdl" as pbcoretools
import "tasks/chunking.wdl" as chunking
import "tasks/pbreports.wdl" as pbreports
import "tasks/pblaa.wdl" as pblaa

workflow pb_laa {
  input {
    File eid_subread
    String dataset_filters = ""
    Int downsample_factor = 0
    Boolean laa_chimera_filter = true
    Boolean laa_clustering = true
    Boolean laa_phasing = true
    Boolean laa_full_length = false
    Int laa_min_length = 3000
    Int laa_max_length = 0
    Int laa_max_reads = 2000
    Float laa_min_snr = 2.5
    Float laa_min_read_score = 0.75
    Int laa_min_barcode_score = 26
    Int laa_max_phasing_reads = 500
    Int laa_ignore_ends = 0

    Int nproc = 8
    Int max_nchunks = 100
    Int? target_size
    String log_level = "DEBUG"
    String? tmp_dir
  }

  call pbcoretools.dataset_absolutize as update_subreads {
    input:
      dataset = eid_subread,
      log_level = log_level,
      nproc = 1
  }

  call pbcoretools.dataset_filter {
    input:
      dataset = update_subreads.updated_xml,
      filters = dataset_filters,
      downsample_factor = downsample_factor,
      log_level = log_level
  }

  call chunking.split_dataset {
    input:
      ds_in = dataset_filter.filtered,
      max_nchunks = max_nchunks,
      target_size = target_size,
      split_mode = "barcodes",
      log_level = log_level
  }

  scatter (chunk_subreads in split_dataset.chunks) {
    call pblaa.laa {
      input:
        subreads = chunk_subreads,
        chimera_filter = laa_chimera_filter,
        clustering = laa_clustering,
        phasing = laa_phasing,
        full_length = laa_full_length,
        min_length = laa_min_length,
        max_length = laa_max_length,
        max_reads = laa_max_reads,
        min_snr = laa_min_snr,
        min_read_score = laa_min_read_score,
        min_barcode_score = laa_min_barcode_score,
        max_phasing_reads = laa_max_phasing_reads,
        ignore_ends = laa_ignore_ends,
        log_level = log_level,
        nproc = nproc
    }
  }

  call chunking.gather_generic as gather_consensus_fastq {
    input:
      chunks = laa.consensus_fastq,
      output_file_name = "amplicon_analysis.fastq"
  }

  call chunking.gather_generic as gather_chimeras_fastq {
    input:
      chunks = laa.chimeras_fastq,
      output_file_name = "amplicon_analysis_chimeras_noise.fastq"
  }

  call chunking.gather_generic as gather_summary_csv {
    input:
      chunks = laa.summary_csv,
      output_file_name = "amplicon_consensus_summary.csv"
  }

  call chunking.gather_generic as gather_input_csv {
    input:
      chunks = laa.input_csv,
      output_file_name = "amplicon_consensus_input.csv"
  }

  call pbreports.amplicon_analysis_consensus {
    input:
      summary_csv = gather_summary_csv.gathered
  }

  call pbreports.amplicon_analysis_input {
    input:
      input_csv = gather_input_csv.gathered
  }

  call pbcoretools.split_laa_fastq {
    input:
      consensus = gather_consensus_fastq.gathered,
      chimeras = gather_chimeras_fastq.gathered,
      subreads = dataset_filter.filtered,
      nproc = nproc,
      log_level = log_level
  }

  call pbcoretools.make_combined_laa_zip {
    input:
      consensus = gather_consensus_fastq.gathered,
      summary_csv = gather_summary_csv.gathered,
      subreads = dataset_filter.filtered,
      nproc = nproc,
      log_level = log_level
  }

  output {
    File consensus_fastq = gather_consensus_fastq.gathered
    File chimeras_fastq = gather_chimeras_fastq.gathered
    File summary_csv = gather_summary_csv.gathered
    File report_consensus = amplicon_analysis_consensus.report
    File report_input = amplicon_analysis_input.report
    File consensus_fastq_split = split_laa_fastq.consensus_fastq_zip
    File chimeras_fastq_split = split_laa_fastq.chimeras_fastq_zip
    File combined_zip = make_combined_laa_zip.combined_zip
  }
}
