# Workflow version of our dataset report generation

version 1.0

import "tasks/pbcoretools.wdl" as pbcoretools
import "tasks/pbreports.wdl" as pbreports

workflow sl_import_subreads {
  input {
    File eid_subread
    Boolean make_reports = true

    Int nproc = 1
    String log_level = "INFO"
    String? tmp_dir
  }

  if (make_reports) {
    call pbcoretools.dataset_absolutize as update_subreads {
      input:
        dataset = eid_subread,
        log_level = log_level,
        nproc = 1
    }

    call pbreports.raw_data {
      input:
        subreads = update_subreads.updated_xml,
        nproc = nproc,
        log_level = log_level
    }
  
    call pbreports.adapters {
      input:
        subreads = update_subreads.updated_xml,
        nproc = nproc,
        log_level = log_level
    }
  
    call pbreports.loading {
      input:
        subreads = update_subreads.updated_xml,
        nproc = nproc,
        log_level = log_level
    }
  
    call pbreports.control {
      input:
        subreads = update_subreads.updated_xml,
        nproc = nproc,
        log_level = log_level
    }

    if (!defined(raw_data.report)) {
      call pbreports.subread_stats {
        input:
          subreads = update_subreads.updated_xml,
          nproc = nproc,
          log_level = log_level
      }
    }
  }

  output {
    File? report_raw_data = raw_data.report
    File? report_adapters = adapters.report
    File? report_loading = loading.report
    File? report_control = control.report
    File? report_subread_stats = subread_stats.report
  }
}
