# Workflow for running auto-merge

version 1.0

import "tasks/sl_tasks.wdl" as sl_tasks
import "tasks/pbcoretools.wdl" as pbcoretools

task dataset_merge {
  input {
    Array[File] datasets
    String dataset_ext
    String dataset_name = "Auto-merged input datasets"
    Boolean skip_counts = false

    Int nproc = 1
    String log_level = "DEBUG"
  }

  command {
    dataset \
      --log-level ${log_level} \
      ${true="--skipCounts" false="" skip_counts} \
      merge \
      --remove-parentage \
      --unique-collections \
      --no-sub-datasets \
      --name "${dataset_name}" \
      merged${dataset_ext} \
      ${sep=" " datasets}
  }
  runtime {
    cpu: 1
  }
  output {
    File merged = "merged${dataset_ext}"
  }
}

workflow sl_merge_datasets {
  input {
    Array[File] datasets
    String dataset_ext
    String dataset_name = "Auto-merged input datasets"
    # FIXME setting this true results in counts of -1, regardless of whether
    # real values are present in the input XMLs
    Boolean skip_counts = false
    Boolean write_report = true

    Int nproc = 1
    String log_level = "DEBUG"
    String? tmp_dir
  }

# disabled this since datasets are already absolutized in SL
# scatter (dataset in datasets) {
#   call pbcoretools.dataset_absolutize {
#     input:
#       dataset = dataset,
#       nproc = nproc,
#       log_level = log_level
#   }
# }

  call dataset_merge {
    input:
#      datasets = dataset_absolutize.updated_xml,
      datasets = datasets,
      dataset_ext = dataset_ext,
      dataset_name = dataset_name,
      skip_counts = skip_counts,
      nproc = nproc,
      log_level = log_level
  }

  if (write_report) {
    # This is here mostly just for testing purposes
    call sl_tasks.simple_dataset_report {
      input:
        dataset = dataset_merge.merged,
        skip_counts = skip_counts,
        nproc = 1,
        log_level = log_level
    }
  }

  output {
    File merged = dataset_merge.merged
    File? report_dataset = simple_dataset_report.report
  }
}
