version 1.0

task build_raw_rdb {
    input {
        File reads_fn
        String db_prefix
        File? blacklist_fofn
        File config_sh_fn
    }
    command {
        set -e
        input_reads_fn="${reads_fn}" \
        input_blacklist_fofn="${blacklist_fofn}" \
        params_config_sh_fn=${config_sh_fn} \
            ipa-task build_raw_rdb

        ln -sf reads.rdb ${db_prefix}.rdb
        ln -sf reads.full.rdb ${db_prefix}.full.rdb
    }
    output {
        File out_rdb = "${db_prefix}.rdb"
        File out_rdb_full = "${db_prefix}.full.rdb"
        File out_cutoff = "length_cutoff"
    }
}

task ovlcns_raw_prepare {
    input {
        String rdb
    }
    command {
        output_blocks=./blocks \
        input_rdb="${rdb}" \
            ipa-task ovlcns_raw_prepare
    }
    output {
        Array[String] blocks = read_lines("./blocks")
    }
}

task ovlcns_raw_run {
    input {
        File rdb
        File length_cutoff_fn
        String db_prefix
        String block_id
        Int nproc
        File config_sh_fn
    }

    command {
        input_rdb="${rdb}" \
        input_length_cutoff_fn="$rel/${length_cutoff_fn}" \
        params_block_id=${block_id} \
        params_num_threads=${nproc} \
        params_config_sh_fn=${config_sh_fn} \
            ipa-task ovlcns_raw_run

        ln -f ovl.m4 ovl.${db_prefix}.${block_id}.m4
        ln -f ovl.flipped.sorted.filtered.m4 ovl.${db_prefix}.flipped.sorted.filtered.${block_id}.m4
        ln -f ercreads.fasta ercreads.${block_id}.fasta
    }
    runtime {
        cpu: nproc
    }

    output {
        File out_m4 = "ovl.rawreads.${block_id}.m4"
        File out_filtered_m4 = "ovl.${db_prefix}.flipped.sorted.filtered.${block_id}.m4"
        File out_ercreads_fasta = "ercreads.${block_id}.fasta"
    }
}

task ovlcns_raw_merge {
    input {
        Array[File] in_fns
    }
    command {
        set -vex
        echo ${sep=' ' in_fns} | xargs -n 1 > ercreads.fofn
        cat ${sep=' ' in_fns} > ercreads.fasta
    }
    output {
        File gathered_fofn = "ercreads.fofn"
        File gathered_fasta = "ercreads.fasta"
    }
}

task build_erc_rdb {
    input {
        File reads_fn
        String db_prefix
        File config_sh_fn
    }

    command {
        input_reads_fn="${reads_fn}" \
        params_db_prefix="${db_prefix}" \
        params_config_sh_fn=${config_sh_fn} \
            ipa-task build_erc_rdb
    }
    output {
        File out_rdb = "${db_prefix}.rdb"
    }
}

task ovl_erc_prepare {
    input {
        String rdb
    }
    command {
        output_blocks=./blocks \
        input_rdb="${rdb}" \
            ipa-task ovl_erc_prepare
    }
    output {
        Array[String] blocks = read_lines("./blocks")
    }
}

task ovl_erc_run {
    input {
        String rdb
        String block_id
        Int nproc
        File config_sh_fn
    }

    command {
        input_rdb="${rdb}" \
        params_block_id=${block_id} \
        params_num_threads=${nproc} \
        params_config_sh_fn=${config_sh_fn} \
            ipa-task ovl_erc_run

        ln -f ovl.m4 ovl.${block_id}.m4
        ln -f ovl.flipped.sorted.dovetail.m4 ovl.flipped.sorted.dovetail.${block_id}.m4
    }
    runtime {
        cpu: nproc
    }
    output {
        File out_m4 = "ovl.${block_id}.m4"
        File out_filtered_m4 = "ovl.flipped.sorted.dovetail.${block_id}.m4"
    }
}

task ovl_erc_merge {
    input {
        Array[File] in_fns
        String db_prefix
        Int nproc
        File config_sh_fn
    }
    command {
        echo "${sep='\n' in_fns}" > ./merged.fofn

        input_fofn=./merged.fofn \
        output_ovl="~{db_prefix}.ovl" \
        params_num_threads="~{nproc}" \
        params_config_sh_fn=${config_sh_fn} \
            ipa-task ovl_erc_merge
    }
    runtime {
        cpu: nproc
    }
    output {
        File gathered_m4 = "${db_prefix}.ovl"
    }
}

task assemble {
    input {
        File in_m4
        File in_ercreads
        String ctg_prefix
        File config_sh_fn
    }

    command {
        input_m4="${in_m4}" \
        input_ercreads="${in_ercreads}" \
        params_ctg_prefix=${ctg_prefix} \
        params_config_sh_fn=${config_sh_fn} \
            ipa-task assemble
    }
    output {
        File p_ctg_fa = "p_ctg.fasta"
        File a_ctg_fa = "a_ctg.fasta"
        File p_ctg_tiling_path = "p_ctg_tiling_path"
        File a_ctg_tiling_path = "a_ctg_tiling_path"
        File asm_gfa = "asm.gfa"
        File sg_gfa = "sg.gfa"
        File contig_gfa2 = "contig.gfa2"
        File circular_contigs = "circular_contigs.csv"
    }
}

task create_datasets {
    input {
        File p_ctg_fa
        File reads_fofn
    }
    command {
        input_fasta="${p_ctg_fa}" \
        input_reads="${reads_fofn}" \
            ipa-task create_datasets
    }
    output {
        File subreadset_xml = "raw.subreadset.xml"
        File contigset_xml = "contigs.referenceset.xml"
    }
}

task oric{
  input{
    File whitelist
    File p_ctg_fa
  }
  command {
    input_fa="${p_ctg_fa}" \
    input_whitelist="${whitelist}" \
    output_fa="p_ctg_oric.fasta" \
    output_json="contigs_report.json" \
        ipa-task circ_oric
  }
  output{
    File p_ctg_oric_fa = "p_ctg_oric.fasta"
    File report = "contigs_report.json"
  }
}
