version 1.0

task lima {
  input {
    File reads
    File barcodes
    Boolean symmetric_barcodes = true
    Boolean peek_guess = true
    Boolean write_unbarcoded = true
    Boolean use_ccs = false
    Int min_score = 0
    Boolean ignore_biosamples = false

    Int nproc
    String log_level = "INFO"
  }
  command {
    lima \
      -j ${nproc} \
      ${true="--peek-guess" false="" peek_guess} \
      ${true="--same" false="--different" symmetric_barcodes} \
      ${"--min-score " + min_score} \
      ${true="--dump-removed" false="" write_unbarcoded} \
      ${true="--ccs" false="" use_ccs} \
      ${true="--ignore-biosamples" false="" ignore_biosamples} \
      --split-bam-named \
      ${reads} \
      ${barcodes} \
      lima.json
  }
  runtime {
    cpu: nproc
  }
  output {
    File datastore = "lima.json"
    File unbarcoded = "lima.removed.bam"
  }
}

# Barcoding with isoseq3 customization
task lima_isoseq {
  input {
    File ccsxml
    File barcodes
    Boolean ignore_biosamples = false

    Int nproc
    String log_level = "INFO"
  }
  # Note the hacky workaround to identify the output file
  command {
    lima \
      -j ${nproc} \
      --isoseq \
      --peek-guess \
      ${true="--ignore-biosamples" false="" ignore_biosamples} \
      ${ccsxml} \
      ${barcodes} \
      fl.datastore.json
  }
  runtime {
    cpu: nproc
  }
  output {
    File datastore = "fl.datastore.json"
  }
}

task update_barcoded_sample_metadata {
  input {
    File lima_datastore
    File input_reads
    File barcodes
    Int min_bq_filter = 26
    Boolean isoseq_mode = false
    Boolean use_barcode_uuids = false

    Int nproc
    String log_level = "DEBUG"
  }
  command {
    python -m pbcoretools.tasks2.update_barcoded_sample_metadata \
      --log-level ${log_level} \
      --nproc ${nproc} \
      ${true="--isoseq-mode" false="" isoseq_mode} \
      ${true="--use-barcode-uuids" false="" use_barcode_uuids} \
      ${"--min-bq-filter " + min_bq_filter} \
      ${input_reads} \
      ${lima_datastore} \
      ${barcodes} \
      metadata_updated.datastore.json
  }
  runtime {
    cpu: nproc
  }
  output {
    File datastore = "metadata_updated.datastore.json"
  }
}

task reparent_dataset {
  input {
    File input_reads
    String dataset_name
    String dataset_ext = ".subreadset.xml"

    Int nproc = 1
    String log_level = "DEBUG"
  }
  command {
    python -m pbcoretools.tasks2.reparent_dataset \
      --log-level ${log_level} \
      ${input_reads} \
      "${dataset_name}" \
      unbarcoded${dataset_ext}
  }
  runtime {
    cpu: 1
  }
  output {
    File new_parent = "unbarcoded${dataset_ext}"
  }
}

task datastore_to_merged_dataset {
  input {
    File datastore
    String dataset_ext = ".subreadset.xml"

    Int nproc = 1
    String log_level = "DEBUG"
  }
  command {
    python -c 'import sys; import os.path ; from pbcommand.models import DataStore ; ds = DataStore.load_from_json(os.path.realpath(sys.argv[1])) ; print "\n".join([f.path for f in ds.files.values()])' ${datastore} > datasets.fofn
    dataset --strict create merged${dataset_ext} datasets.fofn
  }
  runtime {
    cpu: 1
  }
  output {
    File mergedxml = "merged${dataset_ext}"
  }
}
