version 1.0

task ipdsummary {
  input {
    File mapped
    File reference
    Float p_value = 0.001
    String? identify_mods
    Int? max_length
    Boolean? compute_methyl_fraction
    Boolean write_csv = false
    Boolean write_bigwig = true

    Int nproc
    String log_level = "DEBUG"
  }
  # TODO --methylFraction, optional boolean flag
  command {
    ipdSummary \
      --log-level ${log_level} \
      --numWorkers ${nproc} \
      ${mapped} \
      --reference ${reference} \
      --gff basemods.gff \
      ${true="--csv basemods.csv" false="" write_csv} \
      ${true="--bigwig ipds.bw" false="" write_bigwig} \
      --pvalue ${p_value} \
      --alignmentSetRefWindows \
      ${"--identify " + identify_mods} \
      ${"--maxLength " + max_length} \
      ${true="--methylFraction" false="" compute_methyl_fraction}
  }
  runtime {
    cpu: nproc
  }
  output {
    File basemods_gff = "basemods.gff"
    File? basemods_csv = "basemods.csv"
    File? bigwig_file = "ipds.bw"
  }
}

task summarize_modifications {
  input {
    File basemods_gff
    File alignment_gff

    Int nproc = 1
    String log_level = "DEBUG"
  }
  command {
    python \
      -m kineticsTools.summarizeModifications \
      --log-level ${log_level} \
      ${basemods_gff} \
      ${alignment_gff} \
      alignment_summary_with_basemods.gff
  }
  runtime {
    cpu: nproc
  }
  output {
    File summary_basemods_gff = "alignment_summary_with_basemods.gff"
  }
}

task find_motifs {
  input {
    File basemods_gff
    File reference
    Float? min_score

    # FIXME motifMaker doesn't actually allow you to set this 
    Int nproc
    String log_level = "DEBUG"
  }
  # FIXME motifMaker won't accept ReferenceSet directly
  command {
    task_motifmaker_find \
      --log-level ${log_level} \
      ${basemods_gff} \
      ${reference} \
      motifs.csv \
      ${"--minScore " + min_score}
  }
  runtime {
    cpu: nproc
  }
  output {
    File motifs_csv = "motifs.csv"
  }
}

task reprocess_motifs {
  input {
    File basemods_gff
    File motifs_csv
    File reference
    Float? min_fraction

    Int nproc = 1
    String log_level = "DEBUG"
  }
  command {
    task_motifmaker_reprocess \
      --log-level ${log_level} \
      ${basemods_gff} \
      ${motifs_csv} \
      ${reference} \
      motifs.gff \
      ${"--minFraction " + min_fraction}
  }
  runtime {
    cpu: 1
  }
  output {
    File motifs_gff = "motifs.gff"
  }
}

task gather_bigwig {
  input {
    Array[File] bigwig_files

    Int nproc = 1
    String log_level = "DEBUG"
  }
  command {
    python \
      -m kineticsTools.tasks.gather_bigwig \
      --log-level ${log_level} \
      ipds.bw \
      ${sep=" " bigwig_files}
  }
  output {
    File gathered = "ipds.bw"
  }
}

task get_dataset_size {
  input {
    File dataset

    Int nproc = 1
    String log_level = "DEBUG"
  }
  command <<<
    python <<EOF
    from pbcore.io import openDataFile
    ds = openDataFile("~{dataset}")
    with open("length.txt", "w") as txt_out:
      txt_out.write(str(len(ds)))
  >>>
  runtime {
    cpu: 1
  }
  output {
    Int dataset_size = read_int("length.txt")
  }
}
