version 1.0

task train {
  input {
    File alignments
    File reference

    Int nproc
    String log_level = "INFO"
  }

  command {
    hpolish \
      TRAIN \
      --log-file hpolish_train.log \
      --log-level ${log_level} \
      -i ${alignments} \
      -r ${reference} \
      --parameter-file hpolish_train.mat \
  }
  runtime {
    cpu: nproc
  }
  output {
    File matrix = "hpolish_train.mat"
  }
}


task gather_train {
  input {
    Array[File] chunks

    Int nproc = 1
    String log_level = "INFO"
  }

  command {
    hpolish \
      GATHERTRAIN \
      --log-file hpolish_gather_train.log \
      --log-level ${log_level} \
      -i "${sep="," chunks}" \
      --parameter-file hpolish_train.mat
  }
  runtime {
    cpu: 1
  }
  output {
    File gathered = "hpolish_train.mat"
  }
}


task hpolish_call {
  # call is reserved keyword, use hpolish_call instead
  input {
    File alignments
    File reference 
    File matrix

    Int nproc
    Int? minimum_coverage_to_call = 5
    String log_level = "INFO"
  }

  command {
    hpolish \
      CALL \
      --log-file hpolish_call.log \
      --log-level ${log_level} \
      -c ${minimum_coverage_to_call} \
      -i ${alignments} \
      -r ${reference} \
      --stats-file hpolish_call.tsv \
      --parameter-file ${matrix}
  }
  runtime {
    cpu: nproc
  }
  output {
    File stats = "hpolish_call.tsv"
  }
}


task gather_call {
  input {
    Array[File] chunks
    File reference 

    Int nproc = 1
    String log_level = "INFO"
  }

  command {
    hpolish \
      GATHERCALL \
      --log-file hpolish_gather_call.log \
      --log-level ${log_level} \
      -i ${sep="," chunks} \
      -r ${reference} \
      -o hpolish_consensus.fasta \
      --stats-file hpolish_consensus.tsv
  }
  runtime {
    cpu: nproc
  }
  output {
    File gathered = "hpolish_consensus.fasta"
  }
}
