# Components in minor variants analysis

version 1.0

task fuse {
  input {
    File alignments

    Int nproc = 1
  }
  command {
    fuse ${alignments} fuse.fasta
    samtools faidx fuse.fasta
    dataset --strict create --type ReferenceSet --name "Alignment-based reference" fuse.referenceset.xml fuse.fasta
  }
  runtime {
    cpu: nproc
  }
  output {
    File reference_new = "fuse.referenceset.xml"
  }
}

# FIXME can we replace this with pbmm2?
task pbalign_minorvariants {
  input {
    File ccs
    File reference

    Int nproc
    String log_level = "DEBUG"
  }
  command {
    pbalign \
      --log-level ${log_level} \
      --nproc ${nproc} \
      --maxMatch 15 \
      --algorithmOptions '--placeGapConsistently --scoreMatrix "-1 4 4 4 6 4 -1 4 4 6 4 4 -1 4 6 4 4 4 -1 6 6 6 6 6 6"' \
      ${ccs} \
      ${reference} \
      aligned.consensusalignmentset.xml
  }
  runtime {
    cpu: nproc
  }
  output {
    File mapped = "aligned.consensusalignmentset.xml"
  }
}

task cleric {
  input {
    File alignments
    File reference
    File reference_fuse

    Int nproc = 1
  }
  command {
    cleric \
      ${alignments} \
      ${reference} \
      ${reference_fuse} \
      cleric.consensusalignmentset.xml
  }
  runtime {
    cpu: 1
  }
  output {
    File mapped = "cleric.consensusalignmentset.xml"
  }
}

task juliet {
  input {
    File alignments
    String target_config = "none"
    Boolean mode_phasing = false
    String genomic_region = ""
    Float minimal_percentage = 0.1
    Float maximal_percentage = 100
    Boolean only_known_drms = false
    Float substitution_rate = 0.0
    Float deletion_rate = 0.0
    Boolean debug = false

    Int nproc = 1
  }
  # FIXME
  #   ${"--region " + genomic_region} \
  command {
    juliet \
      ${"--config " + target_config} \
      ${true="--mode-phasing" false="" mode_phasing} \
      ${true="--drm-only" false="" only_known_drms} \
      ${"--min-perc " + minimal_percentage} \
      ${"--max-perc " + maximal_percentage} \
      ${"--sub " + substitution_rate} \
      ${"--del " + deletion_rate} \
      ${true="--debug" false="" debug} \
      ${alignments}
  }
  runtime {
    cpu: 1
  }
  output {
    File html = "cleric.html"
    File json = "cleric.json"
  }
}
