version 1.0

# Alignment (should be chunked by ZMWs)
task pbmm2_align {
  input {
    File unmapped
    File? reference
    String aln_ext = ".alignmentset.xml"
    String? preset_mode
    Float min_concordance = 70
    Int min_length = 50
    Boolean hq_mode = false
    Boolean zmw_mode = false
    String? biosample_name
    String? pbmm2_overrides
    Boolean? median_filter = false
    Boolean? strip = false
    Boolean? split_by_sample = false

    Int nproc = 1
    String log_level = "DEBUG"
    String? tmp_dir
  }

  # FIXME restore biosample_name - this may be a Cromwell bug
  # When input only has one biosample, or sample name is overriden by --sample,
  # pbmm2 outputs mapped${aln_ext}.
  # when input has multiple biosample, pbmm2 does not output mapped${aln_ext},
  # instead it outputs multiple bam files and out.json datastore. We need to
  # create mapped${aln_ext}.
  command {
    ${"TMPDIR=" + tmp_dir} \
    pbmm2 \
      align \
      ${reference} \
      ${unmapped} \
      mapped${aln_ext} \
      --sort \
      --min-concordance-perc ${min_concordance} \
      --min-length ${min_length} \
      --sample "${biosample_name}" \
      ${true="--median-filter" false="" median_filter} \
      ${true="--zmw" false="" zmw_mode} \
      ${true="--hqregion" false="" hq_mode} \
      ${"--preset " + preset_mode} \
      ${true="--strip" false="" strip} \
      ${pbmm2_overrides} \
      ${true="--split-by-sample" false="" split_by_sample} \
      -j ${nproc} \
      --log-level ${log_level}
    ls mapped${aln_ext} || ( \
    cp mapped.json mapped.datastore.json
    python \
      -m pbcoretools.tasks2.dataset_to_datastore \
      mapped.datastore.json \
      mapped${aln_ext}
    )
  }
  runtime {
    cpu: nproc
  }
  output {
    File mapped = "mapped${aln_ext}"
  }
}
