version 1.0

task adapters {
  input {
    File subreads

    Int nproc = 1
    String log_level = "INFO"
  }
  command {
    python \
      -m pbreports.report.adapter_xml \
      --log-level ${log_level} \
      --optional \
      ${subreads} \
      adapter.report.json
  }
  runtime {
    backend: "Local"
  }
  output {
    File? report = "adapter.report.json"
  }
}

task loading {
  input {
    File subreads

    Int nproc = 1
    String log_level = "INFO"
  }
  command {
    python \
      -m pbreports.report.loading_xml \
      --log-level ${log_level} \
      --optional \
      ${subreads} \
      loading.report.json
  }
  runtime {
    backend: "Local"
  }
  output {
    File? report = "loading.report.json"
  }
}

task raw_data {
  input {
    File subreads

    Int nproc = 1
    String log_level = "INFO"
  }
  command {
    python \
      -m pbreports.report.filter_stats_xml \
      --log-level ${log_level} \
      --optional \
      ${subreads} \
      raw_data.report.json
  }
  # this one is quite hefty, we don't want it on the head node
  runtime {
    cpu: 1
  }
  output {
    File? report = "raw_data.report.json"
  }
}

task control {
  input {
    File subreads

    Int nproc = 1
    String log_level = "INFO"
  }
  command {
    python \
      -m pbreports.report.control \
      --log-level ${log_level} \
      --optional \
      ${subreads} \
      control.report.json
  }
  runtime {
    backend: "Local"
  }
  output {
    File? report = "control.report.json"
  }
}

task amplicon_analysis_consensus {
  input {
    File summary_csv

    Int nproc = 1
    String log_level = "INFO"
  }

  command {
    python \
      -m pbreports.report.amplicon_analysis_consensus \
      --log-level ${log_level} \
      ${summary_csv} \
      amplicon_analysis_consensus.report.json
  }
  runtime {
    cpu: 1
  }
  output {
    File report = "amplicon_analysis_consensus.report.json"
  }
}

task amplicon_analysis_input {
  input {
    File input_csv

    Int nproc = 1
    String log_level = "INFO"
  }

  command {
    python \
      -m pbreports.report.amplicon_analysis_input \
      --log-level ${log_level} \
      ${input_csv} \
      amplicon_analysis_input.report.json
  }
  runtime {
    cpu: 1
  }
  output {
    File report = "amplicon_analysis_input.report.json"
  }
}

task barcode {
  input {
    File barcoded
    File unbarcoded
    File barcodes

    String report_name = "barcode"
    Boolean test_mode = false
    Int min_bq_filter = 26

    Int nproc = 1
    String log_level = "INFO"
  }

  # FIXME the order here is odd due to the way the Python code is written
  command {
    python \
      -m pbreports.report.${report_name} \
      --log-level ${log_level} \
      --nproc ${nproc} \
      ${true="--test-mode" false="" test_mode} \
      ${"--min-bq-filter " + min_bq_filter} \
      ${barcoded} \
      ${unbarcoded} \
      ${barcodes} \
      barcodes.report.json \
      barcode_summary.csv \
      barcoded_reports.datastore.json
  }
  runtime {
    cpu: nproc
  }
  output {
    File report = "barcodes.report.json"
    File summary_csv = "barcode_summary.csv"
    File reports_datastore = "barcoded_reports.datastore.json"
  }
}

task barcode_isoseq3 {
  input {
    File barcoded
    File unbarcoded
    File barcodes
    File filter_summary
    File flnc_ccs

    # NOTE this doesn't really benefit from parallelization because there
    # should be only one barcoded dataset output right now
    Int nproc = 1
    String log_level = "INFO"
  }

  # FIXME the order here is odd due to the way the Python code is written
  command {
    python \
      -m pbreports.report.barcode_isoseq3 \
      --log-level ${log_level} \
      ${barcoded} \
      ${unbarcoded} \
      ${barcodes} \
      barcode_isoseq3.report.json \
      barcode_isoseq3.report.csv \
      ${filter_summary} \
      ${flnc_ccs}
  }
  runtime {
    cpu: 1
  }
  output {
    File report = "barcode_isoseq3.report.json"
  }
}

task ccs1 {
  input {
    File ccsxml

    Int nproc = 1
    String log_level = "INFO"
  }

  command {
    python \
      -m pbreports.report.ccs \
      --log-level ${log_level} \
      ${ccsxml} \
      ccs.report.json \
      ccs.report.csv
  }
  runtime {
    cpu: 1
  }
  output {
    File report = "ccs.report.json"
    File csv = "ccs.report.csv"
  }
}

task ccs2 {
  input {
    File ccsxml

    Int nproc = 1
    String log_level = "INFO"
  }

  command {
    python \
      -m pbreports.report.ccs2 \
      --log-level ${log_level} \
      ${ccsxml} \
      ccs.report.json \
      -c ccs.report.csv.zip
  }
  runtime {
    cpu: 1
  }
  output {
    File report = "ccs.report.json"
    File csv = "ccs.report.csv.zip"
  }
}

# can also be used for coverage_hgap report
task coverage {
  input {
    File reference
    File coverage_gff
    String report_module = "coverage"

    Int nproc = 1
    String log_level = "INFO"
  }

  command {
    python \
      -m pbreports.report.${report_module} \
      --log-level ${log_level} \
      ${reference} ${coverage_gff} \
      coverage.report.json
  }
  runtime {
    cpu: 1
  }
  output {
    File report = "coverage.report.json"
  }
}

task isoseq3 {
  input {
    File hq_transcripts
    File lq_transcripts

    Int nproc = 1
    String log_level = "INFO"
  }

  command {
    python \
      -m pbreports.report.isoseq3 \
      --log-level ${log_level} \
      ${hq_transcripts} \
      ${lq_transcripts} \
      isoseq3.report.json
  }
  runtime {
    cpu: 1
  }
  output {
    File report = "isoseq3.report.json"
  }
}

task isoseq_mapping {
  input {
    File collapse_fasta_or_fastq
    File collapse_report_json

    Int nproc = 1
    String log_level = "INFO"
  }

  command {
    python \
      -m pbreports.report.isoseq_mapping \
      --log-level ${log_level} \
      ${collapse_fasta_or_fastq} \
      ${collapse_report_json} \
      isoseq_mapping.report.json
  }
  runtime {
    cpu: 1
  }
  output {
    File report = "isoseq_mapping.report.json"
  }
}

# can also be used for mapping_stats_ccs and mapping_stats_hgap variants
task mapping_stats {
  input {
    File mapped
    File? unmapped
    String report_module = "mapping_stats"

    Int nproc = 1
    String log_level = "INFO"
  }

  command {
    python \
      -m pbreports.report.${report_module} \
      --log-level ${log_level} \
      --nproc ${nproc} \
      ${"--unmapped " + unmapped} \
      ${mapped} \
      mapping_stats.report.json
  }
  runtime {
    cpu: nproc
  }
  output {
    File report = "mapping_stats.report.json"
  }
}

task minor_variants {
  input {
    File juliet_json

    Int nproc = 1
    String log_level = "INFO"
  }
  command {
    python \
      -m pbreports.report.minor_variants \
      --log-level ${log_level} \
      ${juliet_json} \
      minor_variants.report.json \
      minor_variants.report.csv
  }
  runtime {
    cpu: 1
  }
  output {
    File report = "minor_variants.report.json"
    File csv = "minor_variants.report.csv"
  }
}

task modifications {
  input {
    File basemods_csv

    Int nproc = 1
    String log_level = "INFO"
  }
  command {
    python \
      -m pbreports.report.modifications \
      --log-level ${log_level} \
      ${basemods_csv} \
      modifications.report.json
  }
  runtime {
    cpu: 1
  }
  output {
    File report = "modifications.report.json"
  }
}

task motifs {
  input {
    File motifs_gff
    File motifs_csv
    Int max_motifs = 10

    Int nproc = 1
    String log_level = "INFO"
  }
  command {
    python \
      -m pbreports.report.motifs \
      --log-level ${log_level} \
      ${motifs_gff} ${motifs_csv} \
      motifs.report.json \
      --maxMotifs ${max_motifs}
  }
  runtime {
    cpu: 1
  }
  output {
    File report = "motifs.report.json"
  }
}

task polished_assembly {
  input {
    File coverage_gff
    File consensus_fastq
    File? report_contigs

    Int nproc = 1
    String log_level = "INFO"
  }
  command {
    python \
      -m pbreports.report.polished_assembly \
      --log-level ${log_level} \
      ${"--contigs-report " + report_contigs} \
      ${coverage_gff} \
      ${consensus_fastq} \
      polished_assembly.report.json
  }
  runtime {
    cpu: 1
  }
  output {
    File report = "polished_assembly.report.json"
  }
}

task site_acceptance_test {
  input {
    File mapped
    # XXX this is hacky, a side effect of making report generation optional
    # to accommodate sub-workflow re-usability
    File? variants_report
    File? mapping_stats_report

    Int nproc = 1
    String log_level = "INFO"
  }

  # FIXME we shouldn't need to specify the output directory
  command {
    python \
      -m pbreports.report.sat \
      --log-level ${log_level} \
      . \
      site_acceptance_test.report.json \
      ${mapped} \
      ${variants_report} \
      ${mapping_stats_report}
  }
  runtime {
    cpu: 1
  }
  output {
    File report = "site_acceptance_test.report.json"
  }
}

task subread_stats {
  input {
    File subreads

    Int nproc = 1
    String log_level = "INFO"
  }
  command {
    python \
      -m pbreports.report.subread_stats \
      --log-level ${log_level} \
      ${subreads} \
      subreads.report.json
  }
  runtime {
    cpu: 1
  }
  output {
    File report = "subreads.report.json"
  }
}

task subreads_reports {
  input {
    File subreads

    Int nproc = 1
    String log_level = "INFO"
  }
  command {
    python \
      -m pbreports.report.subreads_reports \
      --log-level ${log_level} \
      ${subreads} \
      subreads_reports.datastore.json
  }
  runtime {
    cpu: 1
  }
  output {
    File datastore = "subreads_reports.datastore.json"
  }
}

task summarize_coverage {
  input {
    File mapped
    File reference

    Int nproc = 1
    String log_level = "INFO"
  }
  command {
    python \
      -m pbreports.report.summarize_coverage.summarize_coverage \
      --log-level ${log_level} \
      ${mapped} \
      ${reference} \
      coverage.gff
  }
  runtime {
    cpu: 1
  }
  output {
    File coverage_gff = "coverage.gff"
  }
}

task top_variants {
  input {
    File variants_gff
    File reference
    Int nVariants = 100
    Int sortSize = 10000

    Int nproc = 1
    String log_level = "INFO"
  }
  command {
    python \
      -m pbreports.report.top_variants \
      --log-level ${log_level} \
      top_variants.report.json \
      ${variants_gff} ${reference}
  }
  runtime {
    cpu: 1
  }
  output {
    File report = "top_variants.report.json"
  }
}

task variants {
  input {
    File reference
    File consensus_gff
    File variants_gff

    Int nproc = 1
    String log_level = "INFO"
  }
  command {
    python \
      -m pbreports.report.variants \
      --log-level ${log_level} \
      variants.report.json \
      ${reference} ${consensus_gff} ${variants_gff}
  }
  runtime {
    cpu: 1
  }
  output {
    File report = "variants.report.json"
  }
}

task structural_variants_2_report {
  input {
    File lengths_json
    File summary_json

    Int nproc = 1
    String log_level = "INFO"
  }
  command {
    python \
      -m pbreports.report.structural_variants_2 \
      --log-level ${log_level} \
      ${summary_json} \
      ${lengths_json} \
      structural_variants.report.json
  }
  runtime {
    cpu: 1
  }
  output {
    File report = "structural_variants.report.json"
  }
}
