version 1.0

task pbsv_split_ref {
  input {
    File reference
    
    Int nproc = 1
    String log_level = "INFO"
  }
  command {
    python \
      -m pbsvtools.tasks.split_ref_to_chrs \
      --log-level ${log_level} \
      ${reference} \
      pbsv.split_ref.csv
  }
  runtime {
    cpu: nproc
  }
  output {
    File split_ref_csv = "pbsv.split_ref.csv"
  }
}

task pbsv_scatter_tandem_repeat_finder {
  input {
    File split_ref_csv
    Int max_nchunks

    Int? nproc = 1
    String log_level = "INFO"
  }
  command {
    touch .NOARCHIVE
    python \
      -m pbsvtools.tasks.scatter_tandem_repeat_finder2 \
      --log-level ${log_level} \
      ${split_ref_csv} \
      . \
      ${max_nchunks}
  }
  runtime {
    cpu: 1
  }
  output {
    Array[File] chunks = glob("scatter_trf_chunk*.csv")
    Int nchunks = length(chunks)
  }
}

task pbsv_tandem_repeat_finder {
  input {
    File reference
    File split_ref_csv
    
    Int nproc = 1
    String log_level = "INFO"
  }
  command {
    python \
      -m pbsvtools.tasks.tandem_repeat_finder \
      --log-level ${log_level} \
      ${reference} \
      ${split_ref_csv} \
      pbsv.trf.bed
  }
  runtime {
    cpu: nproc
  }
  output {
    File tandem_repeat_bed = "pbsv.trf.bed"
  }
}

task pbsv_gather_tandem_repeat_finder {
  input {
    Array[File] chunks 
    
    Int? nproc = 1
    String? log_level = "INFO"
  }
  command {
    python \
      -m pbsvtools.tasks.gather_bed2 \
      --log-level ${log_level} \
      ${sep=" " chunks} \
      gathered.pbsv.trf.bed
  }
  runtime {
    cpu: 1 
  }
  output {
    File gathered = "gathered.pbsv.trf.bed"
  }
}

task pbsv_scatter_align_datastore {
  input {
    File align_datastore_json
    Int max_nchunks

    Int? nproc = 1
    String? log_level = "INFO"
  }
  command {
    python \
      -m pbsvtools.tasks.scatter_align_datastore \
      --log-level ${log_level} \
      ${align_datastore_json} \
      . \
      ${max_nchunks} \
  }
  runtime {
    cpu: 1
  }
  output {
    Array[File] chunks = glob("chunk*.datastore.json")
    Int nchunks = length(chunks)
  }
}

task pbsv_align_json_to_svsig {
  input {
    File align_datastore_json
    File tandem_repeat_bed
    Int nproc

    String? log_level = "INFO"
  }
  command {
    python \
      -m pbsvtools.tasks.align_json_to_svsig \
      --log-level ${log_level} \
      ${align_datastore_json} \
      ${tandem_repeat_bed} \
      pbsv.svsig.gz.fofn
  }
  runtime {
    cpu: nproc
  }
  output {
    File svsig_gz_fofn = "pbsv.svsig.gz.fofn"
  }
}

task pbsv_call {
  input {
    File reference
    File svsig_gz_fofn
    String chunk_length = "1M"
    Int min_sv_length = 20
    Int min_cnv_length = 1000
    Int min_percent_reads = 20
    Int min_reads_one_sample = 2
    Int min_reads_all_samples = 2
    Int min_reads_per_strand_all_samples = 1

    Int nproc
    String log_level = "INFO"
  }
  command {
    pbsv call \
      --log-level ${log_level} \
      -j ${nproc} \
      --chunk-length ${chunk_length} \
      --min-sv-length ${min_sv_length} \
      --min-cnv-length ${min_cnv_length} \
      --call-min-read-perc-one-sample ${min_percent_reads} \
      --call-min-reads-all-samples ${min_reads_all_samples} \
      --call-min-reads-one-sample ${min_reads_one_sample} \
      --call-min-reads-per-strand-all-samples ${min_reads_per_strand_all_samples} \
      ${reference} \
      ${svsig_gz_fofn} \
      pbsv.vcf \
      structrual_variants.lengths.json \
      structrual_variants.summary.json 
  }
  runtime {
    cpu: nproc
  }
  output {
    File variants = "pbsv.vcf"
    File lengths_report = "structrual_variants.lengths.json"
    File summary_report = "structrual_variants.summary.json"
  }
}


task pbsv_scatter_align_datastore_by_sample {
  input {
    File align_datastore_json
    Int max_nchunks

    Int? nproc
    String? log_level = "INFO"
  }
  command {
    touch .NOARCHIVE
    python \
      -m pbsvtools.tasks.scatter_align_datastore_by_sample \
      --log-level ${log_level} \
      ${align_datastore_json} \
      . \
      ${max_nchunks} 
  }
  runtime {
    cpu: 1
  }
  output {
    Array[File] chunks = glob("chunk_by_sample*.datastore.json")
    Int nchunks = length(chunks)
  }
}


task pbsv_merge_alignments_by_sample {
  input {
    File align_datastore_json
    Int i_chunk = 0

    Int nproc
    String log_level = "INFO"
  }
  command {
    export PB_WORKFLOW_CHUNK_ID=${i_chunk}
    python \
      -m pbsvtools.tasks.merge_alignments_by_sample \
      --nproc ${nproc} \
      --log-level ${log_level} \
      ${align_datastore_json} \
      alignments_by_sample.datastore.json
  }
  runtime {
    cpu: nproc
  }
  output {
    File alignments_by_sample_datastore = "alignments_by_sample.datastore.json"
  }
}


task dataset_to_datastore {
  input {
    File dataset
    Int nproc = 1
    String log_level = "INFO"
  }
  command {
    python \
      -m pbcoretools.tasks2.dataset_to_datastore \
      --log-level ${log_level} \
      ${dataset} \
      dataset_to_datastore.datastore.json
  }
  runtime {
    cpu: nproc
  }
  output {
   File  datastore = "dataset_to_datastore.datastore.json"
  }
}
