# Variant calling using gcpp

version 1.0

import "tasks/chunking.wdl" as chunking
import "tasks/pbreports.wdl" as pbreports
import "tasks/gcpp.wdl" as gcpp

task guess_optimal_max_nchunks {
  input {
    Int nproc
    Int max_nchunks

    String log_level = "INFO"
  }
  Int default_nchunks = 16
  command <<<
    python3 <<EOF
    nproc = ~{nproc}
    nchunks = ~{max_nchunks}
    default_nchunks = min(~{default_nchunks}, nchunks)
    nchunks = max(default_nchunks, 4 * (nchunks // nproc))
    with open("nchunks.txt", "w") as txt_out:
      txt_out.write(str(nchunks))
    EOF
  >>>
  runtime {
    cpu: 1
    backend: "Local"
  }
  output {
    Int nchunks = read_int("nchunks.txt")
  }
}

workflow consensus {
  input {
    File alignments
    File reference
    String consensus_algorithm = "arrow"

    Int nproc = 8
    Int max_nchunks = 100
    Int? target_size
    String log_level = "INFO"
  }

  call guess_optimal_max_nchunks {
    input:
      nproc = nproc,
      max_nchunks = max_nchunks,
      log_level = log_level
  }

  call chunking.split_alignments {
    input:
      mapped = alignments,
      max_nchunks = guess_optimal_max_nchunks.nchunks,
      target_size = target_size,
      log_level = log_level
  }

  scatter (alignment_chunk in split_alignments.chunks) {
    call gcpp.gcpp as genomic_consensus {
      input:
        mapped = alignment_chunk,
        reference = reference,
        consensus_algorithm = consensus_algorithm,
        nproc = nproc,
        log_level = log_level
    }
  }

  call chunking.gather_generic as gather_gff {
    input:
      chunks = genomic_consensus.gff,
      output_file_name = "variants.gff",
      log_level = log_level
  }

  call chunking.gather_generic as gather_vcf {
    input:
      chunks = genomic_consensus.vcf,
      output_file_name = "variants.vcf",
      log_level = log_level
  }

  call chunking.gather_generic as gather_fasta {
    input:
      chunks = genomic_consensus.fasta,
      output_file_name = "consensus.fasta",
      gather_args = "--join-contigs",
      log_level = log_level
  }

  call chunking.gather_generic as gather_fastq {
    input:
      chunks = genomic_consensus.fastq,
      output_file_name = "consensus.fastq",
      gather_args = "--join-contigs",
      log_level = log_level
  }

  output {
    File consensus_fasta = gather_fasta.gathered
    File consensus_fastq = gather_fastq.gathered
    File variants_gff = gather_gff.gathered
    File variants_vcf = gather_vcf.gathered
  }
}
