version 1.0

import "tasks/barcoding.wdl" as barcoding
import "tasks/pbreports.wdl" as pbreports

workflow demultiplex_barcodes {
  input {
    File unbarcoded_reads
    File barcodes
    Boolean lima_symmetric_barcodes = true
    Boolean lima_peek_guess = true
    Int lima_min_score = 0
    Boolean lima_write_unbarcoded = true
    Int min_bq_filter = 26
    Boolean use_barcode_uuids = false
    Boolean lima_use_ccs = false
    Boolean lima_ignore_biosamples = false

    String report_name = "barcode"
    Boolean pb_test_mode = false

    Int nproc
    Int max_nchunks = 1
    String log_level = "INFO"
    Int? target_size
    String? tmp_dir
  }

  call barcoding.lima {
    input:
      reads = unbarcoded_reads,
      barcodes = barcodes,
      symmetric_barcodes = lima_symmetric_barcodes,
      peek_guess = lima_peek_guess,
      min_score = lima_min_score,
      write_unbarcoded = lima_write_unbarcoded,
      use_ccs = lima_use_ccs,
      ignore_biosamples = lima_ignore_biosamples,
      nproc = nproc,
      log_level = log_level
  }

  call barcoding.update_barcoded_sample_metadata {
    input:
      lima_datastore = lima.datastore,
      input_reads = unbarcoded_reads,
      barcodes = barcodes,
      isoseq_mode = false,
      use_barcode_uuids = use_barcode_uuids,
      min_bq_filter = min_bq_filter,
      nproc = nproc,
      log_level = log_level
  }

  call pbreports.barcode as barcode_report {
    input:
      barcoded = update_barcoded_sample_metadata.datastore,
      unbarcoded = unbarcoded_reads,
      barcodes = barcodes,
      report_name = report_name,
      test_mode = pb_test_mode,
      min_bq_filter = min_bq_filter,
      nproc = nproc,
      log_level = log_level
  }

  output {
    File barcoded_reads = update_barcoded_sample_metadata.datastore
    File report_barcodes = barcode_report.report
    File summary_csv = barcode_report.summary_csv
    File barcoded_reports_datastore = barcode_report.reports_datastore
    File unbarcoded = lima.unbarcoded
  }
}
