# HPolish workflow to polish homopolymers of HGAP assembly.
# HPolish is used and only used in HGAP, not resequencing.

version 1.0

import "tasks/hpolish.wdl" as hpolish 
import "tasks/chunking.wdl" as chunking

workflow hpolish {
  input {
    File alignments
    File reference

    Int? minimum_coverage_to_call = 5

    Int nproc = 1
    Int? max_nchunks
    Int? target_size
    String? log_level = "INFO"
  }

  # scatter/gather `hpolish TRAIN`
  call chunking.split_alignments {
    input:
      mapped = alignments,
      max_nchunks = max_nchunks,
      target_size = target_size,
      log_level = log_level 
  }

  scatter (alignments in split_alignments.chunks) {
    call hpolish.train as hpolish_train {
      input:
        alignments = alignments,
        reference = reference,
        nproc = nproc,
        log_level = log_level 
    }
  }
  call hpolish.gather_train as hpolish_gather_train {
    input:
      chunks = hpolish_train.matrix,
      log_level = log_level
  }

  # scatter/gather `hpolish CALL`, reuse split_alignments.chunks
  scatter (alignments in split_alignments.chunks) {
    call hpolish.hpolish_call as hpolish_call {
      input:
        alignments = alignments,
        reference = reference,
        matrix = hpolish_gather_train.gathered,
        nproc = nproc,
        minimum_coverage_to_call = minimum_coverage_to_call,
        log_level = log_level
    }
  }
  call hpolish.gather_call as hpolish_gather_call {
    input:
      chunks = hpolish_call.stats,
      reference = reference,
      log_level = log_level
  }


  output {
    File hpolished_reference = hpolish_gather_call.gathered
  }
}
