# standalone single-sample minor variants workflow

version 1.0

import "tasks/minorvariants.wdl" as minorvariants
import "tasks/pbmm2.wdl" as mapping_tasks
import "tasks/pbreports.wdl" as pbreports

workflow julietflow {
  input {
    File ccs
    File reference
    Boolean juliet_debug = false
    Float deletion_rate = 0.0
    Float minimal_percentage = 0.1
    Float maximal_percentage = 100.0
    Boolean mode_phasing = true
    Boolean only_known_drms = false
    String genomic_region = ""
    Float substitution_rate = 0.0
    String target_config = "none"
    String? pbmm2_overrides

    Int nproc
    String log_level = "DEBUG"
    String? tmp_dir
  }

  call mapping_tasks.pbmm2_align as align1 {
    input:
      unmapped = ccs,
      reference = reference,
      aln_ext = ".consensusalignmentset.xml",
      preset_mode = "CCS",
      pbmm2_overrides = pbmm2_overrides,
      nproc = nproc,
      log_level = log_level,
      tmp_dir = tmp_dir
  }

  call minorvariants.fuse {
    input:
      alignments = align1.mapped,
      nproc = nproc
  }

  call mapping_tasks.pbmm2_align as align2 {
    input:
      unmapped = ccs,
      reference = fuse.reference_new,
      aln_ext = ".consensusalignmentset.xml",
      preset_mode = "CCS",
      nproc = nproc,
      log_level = log_level,
      tmp_dir = tmp_dir
  }

  call minorvariants.cleric {
    input:
      alignments = align2.mapped,
      reference = reference,
      reference_fuse = fuse.reference_new
  }

  call minorvariants.juliet {
    input:
      alignments = cleric.mapped,
      target_config = target_config,
      mode_phasing = mode_phasing,
      genomic_region = genomic_region,
      minimal_percentage = minimal_percentage,
      maximal_percentage = maximal_percentage,
      only_known_drms = only_known_drms,
      substitution_rate = substitution_rate,
      deletion_rate = deletion_rate,
      debug = juliet_debug,
      nproc = nproc
  }

  output {
    File juliet_html = juliet.html
    File juliet_json = juliet.json
    File mapped = align2.mapped
  }
}
